export default [
  require("C:\\Users\\HP\\PycharmProjects\\cognee-eval\\my-website\\node_modules\\infima\\dist\\css\\default\\default.css"),
  require("C:\\Users\\HP\\PycharmProjects\\cognee-eval\\my-website\\node_modules\\@docusaurus\\theme-classic\\lib\\prism-include-languages"),
  require("C:\\Users\\HP\\PycharmProjects\\cognee-eval\\my-website\\node_modules\\@docusaurus\\theme-classic\\lib\\nprogress"),
  require("C:\\Users\\HP\\PycharmProjects\\cognee-eval\\my-website\\src\\css\\custom.css"),
];
