export default {
  "__comp---theme-debug-config-23-a-2ff": [() => import(/* webpackChunkName: "__comp---theme-debug-config-23-a-2ff" */ "@theme/DebugConfig"), "@theme/DebugConfig", require.resolveWeak("@theme/DebugConfig")],
  "__comp---theme-debug-contentba-8-ce7": [() => import(/* webpackChunkName: "__comp---theme-debug-contentba-8-ce7" */ "@theme/DebugContent"), "@theme/DebugContent", require.resolveWeak("@theme/DebugContent")],
  "__comp---theme-debug-global-dataede-0fa": [() => import(/* webpackChunkName: "__comp---theme-debug-global-dataede-0fa" */ "@theme/DebugGlobalData"), "@theme/DebugGlobalData", require.resolveWeak("@theme/DebugGlobalData")],
  "__comp---theme-debug-registry-679-501": [() => import(/* webpackChunkName: "__comp---theme-debug-registry-679-501" */ "@theme/DebugRegistry"), "@theme/DebugRegistry", require.resolveWeak("@theme/DebugRegistry")],
  "__comp---theme-debug-routes-946-699": [() => import(/* webpackChunkName: "__comp---theme-debug-routes-946-699" */ "@theme/DebugRoutes"), "@theme/DebugRoutes", require.resolveWeak("@theme/DebugRoutes")],
  "__comp---theme-debug-site-metadata-68-e-3d4": [() => import(/* webpackChunkName: "__comp---theme-debug-site-metadata-68-e-3d4" */ "@theme/DebugSiteMetadata"), "@theme/DebugSiteMetadata", require.resolveWeak("@theme/DebugSiteMetadata")],
  "__comp---theme-doc-item-178-a40": [() => import(/* webpackChunkName: "__comp---theme-doc-item-178-a40" */ "@theme/DocItem"), "@theme/DocItem", require.resolveWeak("@theme/DocItem")],
  "__comp---theme-doc-roota-94-67a": [() => import(/* webpackChunkName: "__comp---theme-doc-roota-94-67a" */ "@theme/DocRoot"), "@theme/DocRoot", require.resolveWeak("@theme/DocRoot")],
  "__comp---theme-doc-version-roota-7-b-5de": [() => import(/* webpackChunkName: "__comp---theme-doc-version-roota-7-b-5de" */ "@theme/DocVersionRoot"), "@theme/DocVersionRoot", require.resolveWeak("@theme/DocVersionRoot")],
  "__comp---theme-docs-root-5-e-9-0b6": [() => import(/* webpackChunkName: "__comp---theme-docs-root-5-e-9-0b6" */ "@theme/DocsRoot"), "@theme/DocsRoot", require.resolveWeak("@theme/DocsRoot")],
  "__props---docs-11-b-f70": [() => import(/* webpackChunkName: "__props---docs-11-b-f70" */ "@generated/docusaurus-plugin-content-docs/default/p/docs-7fc.json"), "@generated/docusaurus-plugin-content-docs/default/p/docs-7fc.json", require.resolveWeak("@generated/docusaurus-plugin-content-docs/default/p/docs-7fc.json")],
  "__props---docs-docusaurus-debug-content-344-8d5": [() => import(/* webpackChunkName: "__props---docs-docusaurus-debug-content-344-8d5" */ "@generated/docusaurus-plugin-debug/default/p/docs-docusaurus-debug-content-a52.json"), "@generated/docusaurus-plugin-debug/default/p/docs-docusaurus-debug-content-a52.json", require.resolveWeak("@generated/docusaurus-plugin-debug/default/p/docs-docusaurus-debug-content-a52.json")],
  "content---docs-0-e-3-18c": [() => import(/* webpackChunkName: "content---docs-0-e-3-18c" */ "@site/docs/intro.md"), "@site/docs/intro.md", require.resolveWeak("@site/docs/intro.md")],
  "content---docs-getting-started-project-structure-4-c-5-1fd": [() => import(/* webpackChunkName: "content---docs-getting-started-project-structure-4-c-5-1fd" */ "@site/docs/getting-started/project-structure.md"), "@site/docs/getting-started/project-structure.md", require.resolveWeak("@site/docs/getting-started/project-structure.md")],
  "content---docs-getting-started-quickstartfbd-5fc": [() => import(/* webpackChunkName: "content---docs-getting-started-quickstartfbd-5fc" */ "@site/docs/getting-started/quickstart.md"), "@site/docs/getting-started/quickstart.md", require.resolveWeak("@site/docs/getting-started/quickstart.md")],
  "content---docs-getting-started-tutorials-end-to-end-eval-951-e65": [() => import(/* webpackChunkName: "content---docs-getting-started-tutorials-end-to-end-eval-951-e65" */ "@site/docs/getting-started/tutorials/end-to-end eval.md"), "@site/docs/getting-started/tutorials/end-to-end eval.md", require.resolveWeak("@site/docs/getting-started/tutorials/end-to-end eval.md")],
  "content---docs-metrics-generation-alignscore-314-b93": [() => import(/* webpackChunkName: "content---docs-metrics-generation-alignscore-314-b93" */ "@site/docs/metrics/generation/alignscore.md"), "@site/docs/metrics/generation/alignscore.md", require.resolveWeak("@site/docs/metrics/generation/alignscore.md")],
  "content---docs-metrics-generation-bartscore-270-2a7": [() => import(/* webpackChunkName: "content---docs-metrics-generation-bartscore-270-2a7" */ "@site/docs/metrics/generation/bartscore.md"), "@site/docs/metrics/generation/bartscore.md", require.resolveWeak("@site/docs/metrics/generation/bartscore.md")],
  "content---docs-metrics-generation-bertscorefdf-171": [() => import(/* webpackChunkName: "content---docs-metrics-generation-bertscorefdf-171" */ "@site/docs/metrics/generation/bertscore.md"), "@site/docs/metrics/generation/bertscore.md", require.resolveWeak("@site/docs/metrics/generation/bertscore.md")],
  "content---docs-metrics-generation-bleurtscoreff-1-163": [() => import(/* webpackChunkName: "content---docs-metrics-generation-bleurtscoreff-1-163" */ "@site/docs/metrics/generation/bleurtscore.md"), "@site/docs/metrics/generation/bleurtscore.md", require.resolveWeak("@site/docs/metrics/generation/bleurtscore.md")],
  "content---docs-metrics-generation-geval-5-d-5-781": [() => import(/* webpackChunkName: "content---docs-metrics-generation-geval-5-d-5-781" */ "@site/docs/metrics/generation/geval.md"), "@site/docs/metrics/generation/geval.md", require.resolveWeak("@site/docs/metrics/generation/geval.md")],
  "content---docs-metrics-generation-rougescore-0-e-9-008": [() => import(/* webpackChunkName: "content---docs-metrics-generation-rougescore-0-e-9-008" */ "@site/docs/metrics/generation/rougescore.md"), "@site/docs/metrics/generation/rougescore.md", require.resolveWeak("@site/docs/metrics/generation/rougescore.md")],
  "content---docs-metrics-generation-semscore-498-cda": [() => import(/* webpackChunkName: "content---docs-metrics-generation-semscore-498-cda" */ "@site/docs/metrics/generation/semscore.md"), "@site/docs/metrics/generation/semscore.md", require.resolveWeak("@site/docs/metrics/generation/semscore.md")],
  "content---docs-metrics-overviewe-1-d-5c2": [() => import(/* webpackChunkName: "content---docs-metrics-overviewe-1-d-5c2" */ "@site/docs/metrics/overview.md"), "@site/docs/metrics/overview.md", require.resolveWeak("@site/docs/metrics/overview.md")],
  "content---docs-metrics-reranker-cumulative-ndcg-352-591": [() => import(/* webpackChunkName: "content---docs-metrics-reranker-cumulative-ndcg-352-591" */ "@site/docs/metrics/reranker/cumulative-ndcg.md"), "@site/docs/metrics/reranker/cumulative-ndcg.md", require.resolveWeak("@site/docs/metrics/reranker/cumulative-ndcg.md")],
  "content---docs-metrics-reranker-meanap-0-d-2-0b0": [() => import(/* webpackChunkName: "content---docs-metrics-reranker-meanap-0-d-2-0b0" */ "@site/docs/metrics/reranker/map.md"), "@site/docs/metrics/reranker/map.md", require.resolveWeak("@site/docs/metrics/reranker/map.md")],
  "content---docs-metrics-reranker-meanrr-801-b33": [() => import(/* webpackChunkName: "content---docs-metrics-reranker-meanrr-801-b33" */ "@site/docs/metrics/reranker/meanrr.md"), "@site/docs/metrics/reranker/meanrr.md", require.resolveWeak("@site/docs/metrics/reranker/meanrr.md")],
  "content---docs-metrics-reranker-rerank-ndcg-9-d-8-207": [() => import(/* webpackChunkName: "content---docs-metrics-reranker-rerank-ndcg-9-d-8-207" */ "@site/docs/metrics/reranker/reranker-ndcg.md"), "@site/docs/metrics/reranker/reranker-ndcg.md", require.resolveWeak("@site/docs/metrics/reranker/reranker-ndcg.md")],
  "content---docs-metrics-retriever-citationscored-19-4de": [() => import(/* webpackChunkName: "content---docs-metrics-retriever-citationscored-19-4de" */ "@site/docs/metrics/retriever/citationscore.md"), "@site/docs/metrics/retriever/citationscore.md", require.resolveWeak("@site/docs/metrics/retriever/citationscore.md")],
  "content---docs-metrics-retriever-precisionscoreea-5-45d": [() => import(/* webpackChunkName: "content---docs-metrics-retriever-precisionscoreea-5-45d" */ "@site/docs/metrics/retriever/precision.md"), "@site/docs/metrics/retriever/precision.md", require.resolveWeak("@site/docs/metrics/retriever/precision.md")],
  "content---docs-metrics-retriever-recallscorec-14-049": [() => import(/* webpackChunkName: "content---docs-metrics-retriever-recallscorec-14-049" */ "@site/docs/metrics/retriever/recall.md"), "@site/docs/metrics/retriever/recall.md", require.resolveWeak("@site/docs/metrics/retriever/recall.md")],
  "content---docs-metrics-retriever-sufficiencyscore-5-fc-202": [() => import(/* webpackChunkName: "content---docs-metrics-retriever-sufficiencyscore-5-fc-202" */ "@site/docs/metrics/retriever/sufficiencyscore.md"), "@site/docs/metrics/retriever/sufficiencyscore.md", require.resolveWeak("@site/docs/metrics/retriever/sufficiencyscore.md")],
  "plugin---docs-aba-4f5": [() => import(/* webpackChunkName: "plugin---docs-aba-4f5" */ "@generated/docusaurus-plugin-content-docs/default/__plugin.json"), "@generated/docusaurus-plugin-content-docs/default/__plugin.json", require.resolveWeak("@generated/docusaurus-plugin-content-docs/default/__plugin.json")],
  "plugin---docs-docusaurus-debugb-38-c84": [() => import(/* webpackChunkName: "plugin---docs-docusaurus-debugb-38-c84" */ "@generated/docusaurus-plugin-debug/default/__plugin.json"), "@generated/docusaurus-plugin-debug/default/__plugin.json", require.resolveWeak("@generated/docusaurus-plugin-debug/default/__plugin.json")],};
